/*************************************************************************
 *
 **************************************************************************/
#include "includes.h"
#include "System/system.h"

bool cTxos::bFileInit()
{
	if(!bSDMMCReceiveBlock(0x00000000))
	{
		strcpy(ucErrMsg,"File format 0");
		
		return 0;
	}

	if(ucFileBlockData[0] != 0xeb
	   || ucFileBlockData[1] != 0x3c
	   || ucFileBlockData[2] != 0x90)//Jump instruction
	{
		strcpy(ucErrMsg,"File format J");
		
		return 0;
	}
	
	usFileBPS  = ucFileBlockData[11];
	usFileBPS |= ucFileBlockData[12] << 8;
		
	ucFileSPC  = ucFileBlockData[13];
		
	usFileFRS  = ucFileBlockData[14];
	usFileFRS |= ucFileBlockData[15] << 8;
	
	ucFileFAT  = ucFileBlockData[16];
	
	usFileTRE  = ucFileBlockData[17];
	usFileTRE |= ucFileBlockData[18] << 8;
	
	usFileSPF  = ucFileBlockData[22];
	usFileSPF |= ucFileBlockData[23] << 8;
  
	if((ucFileSPC * usFileBPS) != 8192)//Block lengh
	{
		strcpy(ucErrMsg,"Block lengh is not 8192");
		
		return 0;
	}
	
	ucFileMaxBlocksOfCals = (ucFileSPC * usFileBPS)/512;

	//root directory initialize
	if(!bOpenDir(0))
	{
		strcpy(ucErrMsg,"File format-Root dir");
		
		return 0;
	}

	return 1;
}

bool cTxos::bOpenDir(unsigned short usCalusterNumber)
{
	unsigned int uiRamAddr;
	
	//root directory
	uiRamAddr = (usFileFRS+(usFileSPF * ucFileFAT))*usFileBPS;

	if(usCalusterNumber != 0)
		uiRamAddr += ((usCalusterNumber-2)*ucFileSPC*usFileBPS)+(usFileTRE*32);
	
	vDelayUS(10000);
	if(!bSDMMCReceiveBlock(uiRamAddr))
		return 0;

	for(usCalusterNumber=0; usCalusterNumber<512; usCalusterNumber++)
		ucFileDir[usCalusterNumber] = ucFileBlockData[usCalusterNumber];
	
	usFileEntryOffset = 0x0000; //First entry
	
	return 1;
}

void cTxos::vReadDir()
{
	//File name 0x00
	ucFileName[0] = ucFileDir[usFileEntryOffset+0];//0x00:empty - 0xE5:deleted
	ucFileName[1] = ucFileDir[usFileEntryOffset+1];
	ucFileName[2] = ucFileDir[usFileEntryOffset+2];
	ucFileName[3] = ucFileDir[usFileEntryOffset+3];
	ucFileName[4] = ucFileDir[usFileEntryOffset+4];
	ucFileName[5] = ucFileDir[usFileEntryOffset+5];
	ucFileName[6] = ucFileDir[usFileEntryOffset+6];
	ucFileName[7] = ucFileDir[usFileEntryOffset+7];
	ucFileName[8] = '\0';
		
	//File type 0x08
	ucFileType[0] = ucFileDir[usFileEntryOffset+8];
	ucFileType[1] = ucFileDir[usFileEntryOffset+9];
	ucFileType[2] = ucFileDir[usFileEntryOffset+10];
	ucFileType[3] = '\0';

	//File first caluster 0x1A
	usFileOSR  = ucFileDir[usFileEntryOffset+26];
	usFileOSR |= ucFileDir[usFileEntryOffset+27] << 8;
		
	usFileEntryOffset += 0x20; //Next entry

	//End of cache
	if(usFileEntryOffset == 0x200)
		usFileEntryOffset = 0x00; //First entry
}

bool cTxos::bOpenFile(unsigned short usCalusterNumber)
{
	unsigned short usX,
				   usY,
				   usFirstAddr = (usFileFRS*usFileBPS),
				   usLastStart = usFirstAddr;
	
	vDelayUS(100);
	ucFileLastBlockOfCal = 0;
	ucFileLastCal 		 = 0;
	usFileCalusters[0]	 = usCalusterNumber;
	usFileCalusters[255] = 0;
	usY					 = (usFileCalusters[0])*2;
	usFirstAddr			 = (usFileFRS*usFileBPS);
	usLastStart			 = usFirstAddr+(usY&0xfe00);
	
	if(!bSDMMCReceiveBlock(usLastStart))
		return 0;
	
	for(usX=0; usX<255;)
	{
		if(!((usFirstAddr+usY) >= usLastStart && (usFirstAddr+usY) < (usLastStart+512)))
		{			
			usLastStart = usFirstAddr+(usY&0xfe00);
			
			if(!bSDMMCReceiveBlock(usLastStart))
				return 0;
		}
		usY = ucFileBlockData[(usY&0x01ff)]
			  |(ucFileBlockData[(usY&0x01ff)+1]<<8);
		usX++;
		
		if(usY!=0xffff)
			usFileCalusters[usX] = usY;
		else
		{
			usFileCalusters[usX] = 0;
			
			break;
		}
		
		usY = (usFileCalusters[usX])*2;
	}

	return 1;
}

bool cTxos::bReadFile()
{	
	vDelayUS(100);
	if(usFileCalusters[ucFileLastCal]==0)
		return 0;
	
	if(ucFileLastCal == 255)
		if(!bOpenFile(usFileCalusters[ucFileLastCal]))
			return 0;
	
	if(!bSDMMCReceiveBlock((( usFileFRS+(usFileSPF * ucFileFAT)+((usFileCalusters[ucFileLastCal]-2)*ucFileSPC) )*usFileBPS)+(usFileTRE*32)+(ucFileLastBlockOfCal*512)))
		return 0;
	
	if(ucFileLastBlockOfCal < ucFileMaxBlocksOfCals-1)
		ucFileLastBlockOfCal++;
	else
	{
		ucFileLastBlockOfCal = 0;
		ucFileLastCal++;
	}

	return 1;
}

bool cTxos::bWriteFile()
{	
	vDelayUS(100);
	if(usFileCalusters[ucFileLastCal]==0)
		return 0;
	
	if(ucFileLastCal == 255)
		if(!bOpenFile(usFileCalusters[ucFileLastCal]))
			return 0;
	
	if(!bSDMMCSendBlock((( usFileFRS+(usFileSPF * ucFileFAT)+((usFileCalusters[ucFileLastCal]-2)*ucFileSPC) )*usFileBPS)+(usFileTRE*32)+(ucFileLastBlockOfCal*512)))
		return 0;
	
	if(ucFileLastBlockOfCal < ucFileMaxBlocksOfCals-1)
		ucFileLastBlockOfCal++;
	else
	{
		ucFileLastBlockOfCal = 0;
		ucFileLastCal++;
	}

	return 1;
}

void cTxos::vReadFileGoToAddr(unsigned int uiAddr)//512 bytes step - max 2MB offset from 0 in 8192 format
{
	ucFileLastCal 		 = uiAddr/ucFileMaxBlocksOfCals;
	ucFileLastBlockOfCal = uiAddr%ucFileMaxBlocksOfCals;
}
