/*************************************************************************
 *
 **************************************************************************/
#include "includes.h"
#include "System/system.h"

bool cTxos::bSDMMCInit()
{
	unsigned char ucX;
	
	vDelayUS(100000);
	vSDMMCExtraClocks(10);
	vDelayUS(10000);
	
	vSDMMCSendCommand(GO_IDLE_STATE, 0x0, 0x95);
	if(!bSDMMCGetByte(0x01))
	{			
		vDelayUS(100000);
		vSDMMCExtraClocks(100);
		vDelayUS(10000);
		
		vSDMMCSendCommand(GO_IDLE_STATE, 0x0, 0x95);
		if(!bSDMMCGetByte(0x01))
		{	
			vDelayUS(500000);
			vSDMMCExtraClocks(255);
			vSDMMCExtraClocks(255);
			vDelayUS(100000);
			
			vSDMMCSendCommand(GO_IDLE_STATE, 0x0, 0x95);
			if(!bSDMMCGetByte(0x01))
			{
				FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS
				strcpy(ucErrMsg, "SDMMC init");
			
				return 0;
			}
		}
	}
	
	for(ucX=0; ucX<255; ucX++)
	{
		vDelayUS(10000);
		vSDMMCSendCommand(SEND_OP_COND, 0x0, 0xff);
		
		if(bSDMMCGetByte(0x00))
			if(bSDMMCSetBlockLengh(0x00000200))
				return 1;
	}
	FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS
	strcpy(ucErrMsg, "SDMMC init OP_COND");
	
	return 0;
}

bool cTxos::bSDMMCGetByte(unsigned char ucByte)
{
	unsigned short usX;
	unsigned char  ucY = 0xff;
	
	for(usX=0;usX<1000;usX++)
	{
		FIO1PIN_bit.P1_26 = 1;//clock(high=data, low=idle)  CLK
		vDelayUS(SDMMCCLOCKDELAYUS);

		if(FIO1PIN_bit.P1_28)
    		ucY |= 1;//data out			DO
		else
			ucY &= 0xfe;//data out		DO
		
		FIO1PIN_bit.P1_26 = 0;//clock(high=data, low=idle)  CLK
		vDelayUS(SDMMCCLOCKDELAYUS);
		
		if(ucY == ucByte)
			return 1;
		
		ucY *= 2;
	}
	
	return 0;
}

bool cTxos::bSDMMCSetBlockLengh(unsigned int uiLengh)
{
	vSDMMCExtraClocks(1);
	vSDMMCSendCommand(SET_BLOCK_LEN, uiLengh, 0xff);
			
	if(bSDMMCGetByte(0x00))
	{
		FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS
		
		return 1;
	}
	
	FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS
	strcpy(ucErrMsg, "SDMMC block lengh");
	
	return 0;
}

void cTxos::vSDMMCSendCommand(unsigned char ucCmd, unsigned int uiAr, unsigned char ucCrc)
{
	vSDMMCSendByte(ucCmd);
	vSDMMCSendByte(uiAr >> 24);
	vSDMMCSendByte(uiAr >> 16);
	vSDMMCSendByte(uiAr >> 8);
	vSDMMCSendByte(uiAr >> 0);
	vSDMMCSendByte(ucCrc);
}

void cTxos::vSDMMCSendByte(unsigned char ucByte)//ucByte ==> send
{
	unsigned char ucX = 0x80;
	
	while(1)
	{
  		if(ucX&ucByte)
			FIO1PIN_bit.P1_25 = 1;//data in                     DI
		else
			FIO1PIN_bit.P1_25 = 0;//data in                     DI
  
		vDelayUS(SDMMCCLOCKDELAYUS);
		FIO1PIN_bit.P1_26 = 1;//clock(high=data, low=idle)  CLK
	
		vDelayUS(SDMMCCLOCKDELAYUS);
		FIO1PIN_bit.P1_26 = 0;//clock(high=data, low=idle)  CLK
		
		if(ucX==0x01)
			break;
		
		ucX/=2;
	}
}

bool cTxos::bSDMMCSendBlock(unsigned int uiAddr)//ucFileBlockData[0:512] ==> send
{
	unsigned short usY;
	
	vSDMMCExtraClocks(10);
	
	vSDMMCSendCommand(WRITE_BLOCK, uiAddr, 0xff);
	
	if(!bSDMMCGetByte(0x00))
	{
		FIO3PIN_bit.P3_25 = 1;//set SS = 1 (off)            CS
		strcpy(ucErrMsg, "SDMMC write block");
		
		return 0;
	}
	vSDMMCSendByte(0xfe);
	
	for(usY=0; usY<512; usY++)
		vSDMMCSendByte(ucFileBlockData[usY]);
	
	vSDMMCSendByte(0xff);
	vSDMMCSendByte(0xff);
	
	if((ucSDMMCReceiveByte()&0x0F)!=0x05)
	{
		FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS

		return 1;
	}
	FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS
	
	return 0;
}

unsigned char cTxos::ucSDMMCReceiveByte()//receive ==> return
{
	unsigned char ucX, ucData;
	
	for(ucX=0;ucX<8;ucX++)
	{
		ucData *= 2;
		FIO1PIN_bit.P1_26 = 1;//clock(high=data, low=idle)  CLK
		vDelayUS(SDMMCCLOCKDELAYUS);

		if(FIO1PIN_bit.P1_28)
    		ucData |= 1;//data out                    DO
		else
			ucData &= 0xfe;//data out                    DO
		
		FIO1PIN_bit.P1_26 = 0;//clock(high=data, low=idle)  CLK
		vDelayUS(SDMMCCLOCKDELAYUS);
	}
	
	return ucData;
}

bool cTxos::bSDMMCReceiveBlock(unsigned int uiAddr)//receive ==> ucFileBlockData[0:512]
{
	unsigned short usY;
	
	vSDMMCExtraClocks(10);
	
	vSDMMCSendCommand(READ_BLOCK, uiAddr, 0xff);
	
	if(!bSDMMCGetByte(0x00)
	   || !bSDMMCGetByte(0xfe))
	{
		FIO3PIN_bit.P3_25 = 1;//set SS = 1 (off)            CS	
		strcpy(ucErrMsg, "SDMMC read block");
		
		return 0;
	}
	
	for(usY=0; usY<512; usY++)//block=512 byte
		ucFileBlockData[usY] = ucSDMMCReceiveByte();
	
	ucSDMMCReceiveByte();//receive 2 byte after block read
	ucSDMMCReceiveByte();
	
	FIO3PIN_bit.P3_25 =1;//set SS = 1 (off)            CS	
	
	return 1;
}

void cTxos::vSDMMCExtraClocks(unsigned char ucTime)
{
	vDelayUS(10);
	FIO3PIN_bit.P3_25 = 1;//set SS = 1 (off)
	vDelayUS(10);

	for(;ucTime>0;ucTime--)
		vSDMMCSendByte(0xff);//extra clocks to allow mmc to finish off what it is doing

	vDelayUS(10);
	FIO3PIN_bit.P3_25 =0;//set SS = 0 (on)
	vDelayUS(10);
}
