/*************************************************************************
 *
 **************************************************************************/
#include "includes.h"
#include "System/system.h"

void cTxos::vSim908OnOff()
{
	FIO0PIN_bit.P0_24=0;//sim908 on/off
	vDelayUS(1500000);
	FIO0PIN_bit.P0_24=1;//sim908 on/off
}

bool cTxos::bSim908GsmOn()
{
	vHighSpeedOperation();
	
	ucSim908Mode = 1;
	
	//Moudle turning on
	vUartEnable(1);//default baud rate:115200 bps
	vDelayUS(1000);

	U1FCR = 0x07;
	vSim908OnOff();
	vDelayUS(1000);
	
	if(bUartReceive(1))
		;
	
	vDelayUS(500);
	U1FCR = 0x07;
	vUartSend(1,"AT+IPR=1200\r");
	vDelayUS(500);
	//***********************************
	//****Decrease baud rate to 1200*****
	//***********************************
	U1LCR_bit.DLAB = 1;
	U1DLL = 0xd4;
	U1DLM = 0x01;
	U1LCR_bit.DLAB = 0;
	U1FDR_bit.DIVADDVAL = 0;
	vDelayUS(1000);
	
	vWaitForUartResponseMS(1,2000);
	
	if(!bSim908SendCMD("AT+IPR=1200\r",2,1500))
		return 0;
	
	if(!bSim908SendCMD("ATE0\r",2,1500))
		return 0;
	
	if(!bSim908SendCMD("ATX0\r",2,1500))
		return 0;
	
	if(!bSim908SetCall())
		return 0;

	vNormalOperation();

	if(bSim908CheckMode() == 1)
		ucSim908Mode = 0x03;
	else
		return 0;
	
	if(!bSim908SendCMD("AT+CLTS=1\r",2,1500))
		return 0;
	
	return 1;
}

bool cTxos::bSim908GetRegStatus()//1:registered   0:not registered
{
	char ucI, ucJ;
	
	ucSim908Mode &= 0xf3;
	
	if(!bSim908SendCMD("AT+COPS?\r", 2, 1500))
		return 0;
	
	ucI = ucSim908CheckResponceCMD("+COPS: 0,");
	
	if(ucI == 10)
		return 0;
	
	if(ucDataRam[ucI][9] == '0')//+COPS: 0,0,"MTN"
	{
		for(ucJ=0; (ucDataRam[ucI][ucJ+12]!='"' && ucDataRam[ucI][ucJ+12]!='\0'); ucJ++)
			ucDataRam[0][ucJ] = ucDataRam[ucI][ucJ+12];

		ucDataRam[0][ucJ] = '\0';
		ucSim908Mode |= 0x0f;

		return 1;
	}
	
	if(ucDataRam[ucI][9] == '1'
	   || ucDataRam[ucI][9] == '2')
	{
		ucDataRam[0][0]=0;
		ucSim908Mode |= 0x07;
  
		return 1;
	}
	
	return 0;
}

bool cTxos::bSim908Sleep()
{
	if(!bSim908SendCMD("AT+CSCLK=2\r", 2, 1500))
		return 0;

	ucSim908Mode |= 0x80;

	return 1;
}

bool cTxos::bSim908WakeUp()
{
	while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
		;
	
	U1FCR = 0x07;
	vUartSend(1, "AT+CSCLK=0\r");
	vDelayUS(1000);
	
	vWaitForUartResponseMS(1,500);

	while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
		;

	U1FCR = 0x07;
	vUartSend(1, "AT+CSCLK=0\r");
	vDelayUS(1000);
	
	vWaitForUartResponseMS(1,500);
	
	ucSim908Mode &= 0x7f;
	
	return 1;
}

bool cTxos::bSim908SetCall()
{
	ucSim908Mode &= 0x03;

	if(!bSim908SendCMD("AT+CGPSPWR=0\r",2,1500))
		return 0;
	
	if(!bSim908SendCMD("AT+CFUN=1\r",2,1500))
		return 0;

	ucSim908Mode |= 0x03;
	
	return 1;
}

bool cTxos::bSim908GsmOff()
{
	if(!bSim908SendCMD("AT+CFUN=0\r",2, 1500))
		return 0;
	
	ucSim908Mode &= 0xf1;
	
	return 1;
}

bool cTxos::bSim908GpsOn()
{
	ucGPSResetMode = 0;
	
	if(bSim908WakeUp())
		;
	
	if(!bSim908GsmOff())
		return 0;
	
	if(!bSim908SendCMD("AT+CGPSPWR=1\r",2, 1500))// GPS moudle on
		return 0;
	
	if(!bSim908SendCMD("AT+CGPSOUT=0\r",2, 1500))// GPS moudle Out
		return 0;
	
	if(!bSim908SendCMD("AT+CGPSIPR=4800\r",2, 1500))// GPS moudle IPR
		return 0;
	
	if(!bSim908SendCMD("AT+CGPSRST=0\r",2, 1500))// GPS moudle Reset
		return 0;
	
	vDelayUS(100000);
	ucSim908Mode &= 0x71;
	ucSim908Mode |= 0x11;
	
	return 1;
}

bool cTxos::bSim908GpsOff()
{
	if(!bSim908SendCMD("AT+CGPSPWR=0\r",2, 1500))// GPS moudle on
		return 0;
	
	ucSim908Mode &= 0x0f;
	
	return 1;
}

bool cTxos::bSim908GetGPSStatus()
{
	char ucI, ucJ, ucK;
	
	if(!bSim908SendCMD("AT+CGPSSTATUS?\r",2))// GPS moudle INF 
		return 0;
	
	if(bSim908CheckResponce("+CGPSSTATUS: Location 2D Fix"))
	{
		ucSim908Mode &= 0xBF;
		ucSim908Mode |= 0x20;
	}
	else if(bSim908CheckResponce("+CGPSSTATUS: Location 3D Fix"))
	{
		ucSim908Mode &= 0xDF;
		ucSim908Mode |= 0x40;
	}
	else
		return 0;
	
	if(!bSim908SendCMD("AT+CGPSINF=0\r",2))// GPS moudle INF 
				return 0;
	
	ucI = ucSim908CheckResponceCMD("0");
	
	if(ucI == 10)
		return 0;
	
	for(ucJ=2, ucK=0; ucJ<strlen(ucDataRam[ucI]); ucJ++, ucK++)
	{
		if(ucDataRam[ucI][ucJ] == ',')
		{
			ucGPSLong[ucJ] = '\0';
			ucJ++;
			
			for(ucK=0; ucJ<strlen(ucDataRam[ucI]); ucJ++, ucK++)
			{
				if(ucDataRam[ucI][ucJ] == ',')
				{
					ucGPSLat[ucK] = '\0';
					ucJ++;

					for(; ucJ<strlen(ucDataRam[ucI]); ucJ++)
						if(ucDataRam[ucI][ucJ] == ',')
							break;

					ucJ++;

					for(; ucJ<strlen(ucDataRam[ucI]); ucJ++)
						if(ucDataRam[ucI][ucJ] == ',')
							break;

					ucJ++;

					for(; ucJ<strlen(ucDataRam[ucI]); ucJ++)
						if(ucDataRam[ucI][ucJ] == ',')
							break;
					
					ucJ++;
					
					for(; ucJ<strlen(ucDataRam[ucI]); ucJ++)
						if(ucDataRam[ucI][ucJ] == ',')
							break;
					
					ucJ++;
					
					for(ucK=0; ucJ<strlen(ucDataRam[ucI]); ucJ++, ucK++)
					{
						if(ucDataRam[ucI][ucJ] == ',')
						{
							ucGPSSpeed[ucK] = '\0';
					
							goto bGetGPSStatus_1;
						}
						ucGPSSpeed[ucK] = ucDataRam[ucI][ucJ];
					}
				}
				ucGPSLat[ucK] = ucDataRam[ucI][ucJ];
			}
		}
		else
			ucGPSLong[ucK] = ucDataRam[ucI][ucJ];
	}
	
  bGetGPSStatus_1:
	return 1;
}

bool cTxos::bSim908Call(char const* ucData)
{
	char ucNum[30]="ATD";
	
	strcat(ucNum, ucData);
	strcat(ucNum, ";\r");
	
	vDelayUS(500000);
	
	if(!bSim908SendCMD(ucNum,2))
		return 0;
	
	return 1;
}

bool cTxos::bSim908HangUpCall()
{
	if(!bSim908SendCMD("ATH\r",3))
		return 0;
	
	return 1;
}

bool cTxos::bSim908AnswerCall()
{
	if(!bSim908SendCMD("ATA\r",3))
		return 0;
	
	return 1;
}

bool cTxos::bSim908CheckResponce(char const* ucData)
{
	char ucI;
	
	for(ucI=0; ucI<8; ucI++)
		if(strcmp(ucData, ucDataRam[ucI]) == 0)
			return 1;
	
	return 0;
}

bool cTxos::bSim908GetCallingStatus()
{
	char ucI, ucJ;
	
	if(!bSim908SendCMD("AT+CLCC\r",2))//
		return 0;

	ucI = ucSim908CheckResponceCMD("+CLCC:");
	
	if(ucI == 10)
		return 0;

	if(ucDataRam[ucI][8] == ',')
	{
		ucDataRam[1][0] = ucDataRam[ucI][11];
		ucDataRam[1][1] = '\0';
		
		for(ucJ=0; (ucDataRam[ucI][ucJ+18]!='"' && ucDataRam[ucI][ucJ+18]!='\0'); ucJ++)
			ucDataRam[0][ucJ] = ucDataRam[ucI][ucJ+18];
			
		ucDataRam[0][ucJ] = '\0';
			
		return 1;
	}
	
	return 0;
}

bool cTxos::bSim908CheckMode()
{
	if(bSim908SendCMD("AT\r",0,1000))
		if(bSim908CheckResponce("OK"))
			return 1;
	
	if(bSim908SendCMD("AT\r"))
		if(bSim908CheckResponce("OK"))
			return 1;
	
	if(bSim908SendCMD("AT\r"))
		if(bSim908CheckResponce("OK"))
			return 1;
	
	ucSim908Mode = 0;
		
	return 0;
}

bool cTxos::bSim908SendCMD(char const* ucData, unsigned char ucRe, unsigned short usWFR)
{
  _bSim908SendCMD_1:
	while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
		;
	
	U1FCR = 0x07;
	vUartSend(1, ucData);
	vDelayUS(1000);
	
	vWaitForUartResponseMS(1,usWFR);
	
	if(ucSim908Mode == 0)
	{
		if(ucRe == 0)
			return 0;
		
		ucRe--;
		
		goto _bSim908SendCMD_1;
	}
	
	return 1;
}

unsigned char cTxos::ucSim908CheckResponceCMD(char const* ucData)
{
	char ucI, ucJ;
	
	for(ucI=0; ucI<8; ucI++)
	{
		for(ucJ=0; ucJ<strlen(ucData); ucJ++)
		{
			if(ucDataRam[ucI][ucJ] != ucData[ucJ])
				break;
		}
		
		if(ucJ == strlen(ucData))
			return ucI;
	}
	
	return 10;
}

void cTxos::vSim908ShowTimeBATAnt()
{
	char ucI, ucJ;
	//TIME
	if(bSim908SendCMD("AT+CCLK?\r",2,1500))
	{
		ucI = ucSim908CheckResponceCMD("+CCLK: ");
		
		if(ucI != 10)
		{
			for(ucJ=0; ucJ<17; ucJ++)
				ucSystemLog[ucJ] = ucDataRam[ucI][ucJ+8];
			
			ucSystemLog[17] = 0;
			vSetPBGCS(16, 0xffff, 0x0000, 1);
			vSPrint(ucSystemLog);
		}
	}
	//BATTERY
	if(bSim908SendCMD("AT+CBC\r",2,1000))
	{
		ucI = ucSim908CheckResponceCMD("+CBC: ");
		
		if(ucI != 10)
		{
			if(ucDataRam[ucI][6] == '1' || ucDataRam[ucI][6] == '2')
			{
				vSetPBGCS(13, 0xffff, 0x0000, 1);
				vPrint((0x20+160));
			}
			
			if((ucDataRam[ucI][8] == '1'
			    && ucDataRam[ucI][9] == '0'
			    && ucDataRam[ucI][10] == '0')
			   || (ucDataRam[ucI][8] == '9'
			       && ucDataRam[ucI][10] == ','))
			{
				vSetPBGCS(12, 0xffff, 0x07E0, 1);
				vPrint((0x20+159));
			}
			else if((ucDataRam[ucI][8] == '8'
			         && ucDataRam[ucI][10] == ',')
					|| (ucDataRam[ucI][8] == '7'
			            && ucDataRam[ucI][10] == ','))
			{
				vSetPBGCS(12, 0xffff, 0x07E0, 1);
				vPrint((0x20+158));
			}
			else if((ucDataRam[ucI][8] == '6'
			         && ucDataRam[ucI][10] == ',')
					|| (ucDataRam[ucI][8] == '5'
			            && ucDataRam[ucI][10] == ','))
			{
				vSetPBGCS(12, 0xffff, 0x07E0, 1);
				vPrint((0x20+157));
			}
			else if((ucDataRam[ucI][8] == '4'
			         && ucDataRam[ucI][10] == ',')
					|| (ucDataRam[ucI][8] == '3'
			            && ucDataRam[ucI][10] == ','))
			{
				vSetPBGCS(12, 0xffff, 0x07E0, 1);
				vPrint((0x20+156));
			}
			else
			{
				vSetPBGCS(12, 0xffff, 0xf800, 1);
				vPrint((0x20+155));
			}
		}
	}
	//Antenna
	if(bSim908SendCMD("AT+CSQ\r",2,1000))
	{
		ucI = ucSim908CheckResponceCMD("+CSQ: ");
		
		if(ucI != 10)
		{
			vSetPBGCS(14, 0xffff, 0x0000, 1);
			vPrint((0x20+151));
			
			if(ucDataRam[ucI][6] == '0'
			   || (ucDataRam[ucI][6] == '9'
				   && ucDataRam[ucI][7] == '9'))
			{
				vSetPBGCS(15, 0xffff, 0x0000, 1);
				vPrint(' ');
			}
			
			if((ucDataRam[ucI][6] == '2'
			    && ucDataRam[ucI][8] == ',')
			   || (ucDataRam[ucI][6] == '3'
			       && ucDataRam[ucI][8] == ','))
			{
				vSetPBGCS(15, 0xffff, 0x0000, 1);
				vPrint((0x20+154));
			}
			else if(ucDataRam[ucI][6] == '1'
					&& ucDataRam[ucI][8] == ',')
			{
				vSetPBGCS(15, 0xffff, 0x0000, 1);
				vPrint((0x20+153));
			}
			else
			{
				vSetPBGCS(15, 0xffff, 0xf800, 1);
				vPrint((0x20+152));
			}
		}
	}
}

bool cTxos::bSim908SendSMS()
{
	unsigned int  uiITOHexChar;
	unsigned char ucI;
	
	if(ucSMSPrintNumberPos> 0 && ucSim908Mode != 0)
	{
		while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
			;
	
		U1FCR = 0x07;
		
		//vUartSend(1, ucData);
		//{
			//AT+CMGS=""\r
			U1THR = 'A';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = 'T';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = '+';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = 'C';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = 'M';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = 'G';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = 'S';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = '=';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = '"';
			uiITOHexChar = uiITOHex(ucPhoneNumber[0]);
			while(U1LSR_bit.THRE == 0)
				;
			
			for(ucI=0; ucI<ucSMSPrintNumberPos;)
			{
				
				U1THR = uiITOHexChar>>8;
				while(U1LSR_bit.THRE == 0)
					;
				
				U1THR = uiITOHexChar>>0;
				ucI++;
				uiITOHexChar = uiITOHex(ucPhoneNumber[ucI]);
				while(U1LSR_bit.THRE == 0)
					;
			}
			
			U1THR = '"';
			while(U1LSR_bit.THRE == 0)
				;
			U1THR = '\r';
			while(U1LSR_bit.THRE == 0)
				;
		//}

		vDelayUS(1000);
		vWaitForUartResponseMS(1,1000);
		
		if(ucDataRam[0][0] != '>')
			return 0;
		
		while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
			;
	
		U1FCR = 0x07;
		
		//vUartSend(1, ucData);
		//{
			//... <Ctrl+Z>
			uiITOHexChar = uiITOHex(ucSystemLog[0]);

			for(ucI=0; ucI<ucSMSPrintCharNumber;)
			{
				U1THR = uiITOHexChar>>24;
				while(U1LSR_bit.THRE == 0)
					;
				
				U1THR = uiITOHexChar>>16;
				while(U1LSR_bit.THRE == 0)
					;
				
				U1THR = uiITOHexChar>>8;
				while(U1LSR_bit.THRE == 0)
					;
				
				U1THR = uiITOHexChar>>0;
				ucI++;
				uiITOHexChar = uiITOHex(ucSystemLog[ucI]);
				while(U1LSR_bit.THRE == 0)
					;
			}
			
			U1THR = 0x1A;
			while(U1LSR_bit.THRE == 0)
				;
		//}

		vDelayUS(1000);
		vWaitForUartResponseMS(1,6000);
		
		if(ucDataRam[0][0] != 'E')
			return 1;
	}
	
	return 0;
}

bool cTxos::bSim908ReadSMS()
{
	unsigned int uiTime, uiCh;
	unsigned char ucI, ucX, ucJ, ucY, ucCh;
	bool bUnRead = 0;

	for(ucI=0; ucI<9; ucI++)
	{
		ucJ = 0;
		ucY = 0;
		ucCh = 0;
		uiCh = 0;
		uiTime = 4000;
		bUnRead = 0;
		ucSystemLog[0] = 'A';
		ucSystemLog[1] = 'T';
		ucSystemLog[2] = '+';
		ucSystemLog[3] = 'C';
		ucSystemLog[4] = 'M';
		ucSystemLog[5] = 'G';
		ucSystemLog[6] = 'R';
		ucSystemLog[7] = '=';
		ucSystemLog[9] = '\r';
		ucSystemLog[10] = 0;
		//bSim908SendCMD
		while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
			;
		
		U1FCR = 0x07;
		vUartSend(1, ucSystemLog);
		ucX = U1RBR;
		ucX = U1RBR;
		vDelayUS(500);
		
		//vWaitForUartResponseMS
		ucDataRam[0][0]='\0';
		ucDataRam[1][0]='\0';
		ucDataRam[2][0]='\0';
		ucDataRam[3][0]='\0';
		ucDataRam[4][0]='\0';

		while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
			;

		for(uiTime*=10; uiTime!=0; uiTime--)
		{
			vDelayUS(100);
		
			if(U1LSR & 0x01)
			{
				ucX = U1RBR;
				
				if(ucX == 0x0a || ucX == 0x0d)
				{
					if(ucDataRam[ucY][0] != '\0')
					{
						if(ucY == 0)
						{
							uiTime = 1;
							ucI = 9;
						}
						else if(ucY == 4)
						{
							if(ucSystemLog[8] == '9')
								ucSystemLog[8] = '1';
							else
								ucSystemLog[8]++;

							return 1;
						}
						ucY++;
						ucCh = 0;
						uiCh = 0;
					}
				}
				else if(ucX<128 && ucX>6)
				{
					if(ucY == 0)
					{
						if(ucJ == 12 && (ucX == 'U' || ucX == 'N'))
						{
							bUnRead = 1;
							bNewSMS = 0;
						}
						else if(ucJ == 0 && (ucX == 'E' || ucX == 'A'))
						{
							uiTime = 1;
							ucI = 9;
						}
						else if(ucJ == 1 && ucX == 'K')
						{
							vDelayUS(100000);
							uiTime = 1;
						}
						else if((ucJ > 20 && bUnRead==1)||(ucJ > 18 && bUnRead==0))
						{
							if(ucX == '"')
							{
								if(bUnRead==1)
								{
									ucDataRam[0][strlen(ucDataRam[0])+1] = 0;
									ucDataRam[0][strlen(ucDataRam[0])] = '*';
								}
							}
							else if(ucX == ',')
								ucY++;
							else
							{
								uiCh |= (ucX<<((ucCh+2)*8));

								if(ucCh == 1)
								{
									ucDataRam[0][strlen(ucDataRam[0])+1] = 0;
									ucDataRam[0][strlen(ucDataRam[0])] = ucHexToI(uiCh);
									ucCh = 0;
									uiCh = 0;
								}
								else
									ucCh++;
							}
						}
					}
					else if(ucY == 1)
					{
						if(ucX == ',')
							ucY=6;
						else if(ucX != '"')
						{
							uiCh |= (ucX<<(ucCh*8));

							if(ucCh == 3)
							{
								ucJ = strlen(ucDataRam[1]);
								ucDataRam[1][ucJ] = ucHexToI(uiCh);
								ucDataRam[1][ucJ+1] = '\0';
								ucCh = 0;
								uiCh = 0;
							}
							else
								ucCh++;
						}
					}
					else if(ucY == 6)
						ucY = 2;
					else if(ucY == 2)
					{
						if(ucX != '"' && ucX != '+')
						{
							ucJ = strlen(ucDataRam[2]);
							ucDataRam[2][ucJ] = ucX;
							ucDataRam[2][ucJ+1] = '\0';
						}
						else
							ucY++;
					}
					else if(ucY == 3)
					{
						ucJ = strlen(ucDataRam[3]);
						ucDataRam[3][ucJ] = ucX;
						ucDataRam[3][ucJ+1] = '\0';
					}
					else
					{
						uiCh |= (ucX<<(ucCh*8));
						
						if(ucCh == 3)//Hex decode
						{
							ucJ = strlen(ucDataRam[4]);
							ucDataRam[4][ucJ] = ucHexToI(uiCh);
							ucDataRam[4][ucJ+1] = '\0';
							ucCh = 0;
							uiCh = 0;
						}
						else if(ucCh == 1 && uiCh != 0x00003030 && uiCh != 0x00003630)//GSM decode
						{
							uiCh = uiCh<<16;
							ucJ = strlen(ucDataRam[4]);
							ucDataRam[4][ucJ] = ucHexToI(uiCh);
							ucDataRam[4][ucJ+1] = '\0';
							ucCh = 0;
							uiCh = 0;
						}
						else
							ucCh++;
					}
					ucJ++;
				}
			}
		}

		if(ucSystemLog[8] == '9')
			ucSystemLog[8] = '1';
		else
			ucSystemLog[8]++;
	}
	vDelayUS(1000);

	return 0;
}