/*************************************************************************
 *
 **************************************************************************/
#include "includes.h"

/*************************************************************************
 * Function Name: NVIC_IntEnable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Enable interrup at NVIC
 *
 *
 *************************************************************************/
void NVIC_IntEnable(unsigned int IntNumber)
{
	volatile unsigned long * pNVIC_SetEn = &SETENA0;

	assert((NVIC_WDT <= IntNumber) && (NVIC_PLL1 >= IntNumber));
	IntNumber -= NVIC_WDT;

	pNVIC_SetEn += IntNumber/32;
	*pNVIC_SetEn = (1UL<<(IntNumber%32));
}
/*************************************************************************
 * Function Name: NVIC_IntDisable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Disables interrup at NVIC
 *
 *
 *************************************************************************/
void NVIC_IntDisable(unsigned int IntNumber)
{
	volatile unsigned long * pNVIC_ClrEn = &CLRENA0;

	assert((NVIC_WDT <= IntNumber) && (NVIC_PLL1 >= IntNumber));
	IntNumber -= NVIC_WDT;

	pNVIC_ClrEn += IntNumber/32;
	*pNVIC_ClrEn = (1UL<<(IntNumber%32));
}
/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description:Clear pending interrupt at NVIC
 *
 *
 *************************************************************************/
void NVIC_ClrPend(unsigned int IntNumber)
{
	volatile unsigned long * pNVIC_ClrPend = &CLRPEND0;

	assert((NVIC_WDT <= IntNumber) && (NVIC_PLL1 >= IntNumber));
	IntNumber -= NVIC_WDT;

	pNVIC_ClrPend += IntNumber/32;
	*pNVIC_ClrPend = (1UL<<(IntNumber%32));
}
/*************************************************************************
 * Function Name: NVIC_SetPend
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description:Set pending interrupt at NVIC
 *
 *
 *************************************************************************/
void NVIC_SetPend(unsigned int IntNumber)
{
	volatile unsigned long * pNVIC_SetPend = &SETPEND0;

	assert((NVIC_WDT <= IntNumber) && (NVIC_PLL1 >= IntNumber));
	IntNumber -= NVIC_WDT;

	pNVIC_SetPend += IntNumber/32;
	*pNVIC_SetPend = (1UL<<(IntNumber%32));
}
/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number, Interrupt Priority
 * Return: void
 *
 * Description:Sets Interrupt priority
 *
 *
 *************************************************************************/
void NVIC_IntPri(unsigned int IntNumber, char Priority)
{
	volatile char * pNVIC_IntPri = (char *)&IP0;

	assert((NVIC_WDT <= IntNumber) && (NVIC_PLL1 >= IntNumber));
	IntNumber -= NVIC_WDT;
	pNVIC_IntPri += IntNumber;
	*pNVIC_IntPri = Priority;
}