/*************************************************************************
 *
 **************************************************************************/
#include "includes.h"
#include "System/system.h"

//CLOCK&POWER Modes
void cTxos::vHighSpeedOperation()//18MHz
{
	vDelayUS(1000);
	FLASHCFG_bit.FLASHTIM = 0x3; //MIPS = 18.6/1 = 18.6mips
	vDelayUS(1000);
	// 1. Disconnect PLL
	PLL0CON_bit.PLLC = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 2. Disable PLL
	PLL0CON_bit.PLLE = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 3. Select source clock for PLL
	PCLKSEL0 = 0xffffffff;
	PCLKSEL1 = 0xffffffff;   // other peripherals 72/8 = 9 mhz
	
	// 4. Enable PLL
	PLL0CON_bit.PLLE = 1;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 5. Wait for the PLL to achieve lock
	while(!PLL0STAT_bit.PLOCK);
	
	// 6. Set clk divider settings
	CCLKCFG   = 3-1;            //cclk = 72 mhz (2*4*27/1)/3
	
	// 7. Connect the PLL
	vDelayUS(100); 
	PLL0CON_bit.PLLC = 1;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	ucSystemPowerMode = 0x03;
}

void cTxos::vNormalOperation()//4MHz
{
	// 1. Disconnect PLL
	PLL0CON_bit.PLLC = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 2. Disable PLL
	PLL0CON_bit.PLLE = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 3. Select source clock for PLL
	PCLKSEL0 = 0xfd7ffdf7;//df7
	PCLKSEL1 = 0xdfffffff;   // other peripherals 4/8 = 500 KHz
	
	// 4. Set clk divider settings
	CCLKCFG   = 1-1;            //cclk = 4/1 mhz
	
	ucSystemPowerMode = 0x02;
	
	vDelayUS(1000);
	FLASHCFG_bit.FLASHTIM = 0x0;
	//UART 1200
	vDelayUS(100);
	U1LCR_bit.DLAB = 1;
	U1DLL = 0xd4;
	U1DLM = 0x00;
	U1LCR_bit.DLAB = 0;
	U1FDR_bit.DIVADDVAL = 1;
	U1FDR_bit.MULVAL = 14;
	vDelayUS(1000);
}

void cTxos::vSleep()//4MHz/6
{
	// 1. Disconnect PLL
	PLL0CON_bit.PLLC = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 2. Disable PLL
	PLL0CON_bit.PLLE = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 3. Select source clock for PLL
	PCLKSEL0 = 0xfd7ffdf7;
	PCLKSEL1 = 0xffffffff;// other peripherals 4/8 = 500/8 KHz
	
	// 4. Set clk divider settings
	CCLKCFG   = 8-1;//cclk = 4/8 mhz
	
	vDelayUS(1000);
	FLASHCFG_bit.FLASHTIM = 0x5;
	//UART 1200
	vDelayUS(100);
	U1LCR_bit.DLAB = 1;
	U1DLL = 0x1b;
	U1DLM = 0x00;
	U1LCR_bit.DLAB = 0;
	U1FDR_bit.DIVADDVAL = 0;
	ucSystemPowerMode = 0x01;
	vDelayUS(1000);
}

void cTxos::vPowerDown()//15KHz/6
{
	// 1. Disconnect PLL
	PLL0CON_bit.PLLC = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 2. Disable PLL
	PLL0CON_bit.PLLE = 0;
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	
	// 3. Set clk divider settings
	CCLKCFG   = 256-1;            //cclk = 4/256 mhz
	
	vDelayUS(1000);
	FLASHCFG_bit.FLASHTIM = 0x5;
	ucSystemPowerMode = 0x00;
	vDelayUS(1000);
}

//GPIO
void cTxos::vGpioInit()
{
	// Set to inputs
	FIO0DIR = 0x00000000;
	FIO1DIR = 0x00000000;
	FIO2DIR = 0x00000000;
	FIO3DIR = 0x00000000;
	FIO4DIR = 0x00000000;
  
	FIO0CLR = 0xffffffff;
	FIO1CLR = 0xffffffff;
	FIO2CLR = 0xffffffff;
	FIO3CLR = 0xffffffff;
	FIO4CLR = 0xffffffff;

	// clear mask registers
	FIO0MASK = 0x00000000;
	FIO1MASK = 0x00000000;
	FIO2MASK = 0x00000000;
	FIO3MASK = 0x00000000;
	FIO4MASK = 0x00000000;

	// Reset all GPIO pins to default primary function
	PINSEL0 = 0x00000000;//0 0-15
	PINSEL1 = 0x00000000;//0 16-31
	PINSEL2 = 0x00000000;//1 0-15
	PINSEL3 = 0x00000000;//1 16-31
	PINSEL4 = 0x00000000;//2 0-15
	PINSEL7 = 0x00000000;//3 16-31
	PINSEL8 = 0x00000000;//4 0-15
	PINSEL9 = 0x00000000;//4 16-31
  
	PINMODE0 = 0x00000000;
	PINMODE1 = 0x28228300;
	PINMODE2 = 0x00000000;
	PINMODE3 = 0x08020000;
	PINMODE4 = 0x00080000;
	PINMODE5 = 0x00000000;
	PINMODE6 = 0x00000000;
	PINMODE7 = 0x00200000;
	PINMODE8 = 0x00000000;
	PINMODE9 = 0x00000000;
	
	PINMODE_OD0 = 0x00000000;
	PINMODE_OD1 = 0x00000000;
	PINMODE_OD2 = 0x00000000;
	PINMODE_OD3 = 0x00000000;
	PINMODE_OD4 = 0x00000000;
  
	FIO0DIR0 = 0xff;//0:7  lcd data 0:7
	FIO0PIN0 = 0x00;//0:7  lcd data 0:7
	FIO2DIR0 = 0xff;//8:15 lcd data 8:15
	FIO2PIN0 = 0x00;//8:15 lcd data 8:15
	
	FIO0DIR_bit.P0_29=0;//D+
	FIO0DIR_bit.P0_30=0;//D-
	FIO2DIR_bit.P2_9 =0;//USB connect
	
	FIO0DIR_bit.P0_24=1;//sim908 on/off
	FIO0PIN_bit.P0_24=1;//sim908 on/off
	
	FIO1DIR_bit.P1_18=1;//Green LED
	FIO1PIN_bit.P1_18=0;//Green LED
	
	FIO0DIR_bit.P0_9 =1;//LCD light
	FIO0PIN_bit.P0_9 =0;//LCD light
	
	FIO0DIR_bit.P0_19=1;//lcd CS#
	FIO1DIR_bit.P1_19=1;//lcd RS
	FIO1DIR_bit.P1_20=1;//lcd WR#
	FIO1DIR_bit.P1_21=1;//lcd RD#
	FIO1DIR_bit.P1_22=1;//lcd RESET#
	
	FIO0PIN_bit.P0_19=1;//lcd CS#
	FIO1PIN_bit.P1_19=1;//lcd RS
	FIO1PIN_bit.P1_20=1;//lcd WR#
	FIO1PIN_bit.P1_21=1;//lcd RD#
	FIO1PIN_bit.P1_22=1;//lcd RESET#
	
	FIO3DIR_bit.P3_25 = 1;//SDMMC set SS = 1 (off)            CS
	FIO1DIR_bit.P1_25 = 1;//SDMMC data in                     DI
	FIO1DIR_bit.P1_26 = 1;//SDMMC clock(high=data, low=idle)  CLK
	FIO1DIR_bit.P1_28 = 0;//SDMMC data out                    DO
	
	FIO1DIR_bit.P1_29=0;//Headphone
	FIO1PIN_bit.P1_29=1;//Headphone
	
	FIO3DIR_bit.P3_26=0;//Speaker
	FIO3PIN_bit.P3_26=1;//Speaker
	
	//Touch screen init
	FIO1DIR_bit.P1_23=0;//lcd touch screen x+ 
	FIO0DIR_bit.P0_25=0;//lcd touch screen x- ad2
	FIO1DIR_bit.P1_24=1;//lcd touch screen y+
	FIO0DIR_bit.P0_23=1;//lcd touch screen y- ad0
  
	FIO1PIN_bit.P1_23=0;//lcd touch screen x+ 
	FIO0PIN_bit.P0_25=0;//lcd touch screen x- ad2
	FIO1PIN_bit.P1_24=0;//lcd touch screen y+
	FIO0PIN_bit.P0_23=0;//lcd touch screen y- ad0
}

//TIMER
void cTxos::vDelayUS(unsigned int uiValue)
{
	if(uiValue!=0)
	{
		if(ucSystemPowerMode == 0x03)
		{
			T0PR = 9-1; //9mhz pclk
			T0TCR = 0x1;
		
			T0PC = 0;
			T0TC = 0;
			
			while(T0TC <= uiValue)
				;
		}
		else if(ucSystemPowerMode == 0x00)
		{
			uiValue = uiValue/1000;
			T0PR = 15-1; //1000 Hz
			T0TCR = 0x1;
		
			T0PC = 0;
			T0TC = 0;
			
			while(T0TC <= uiValue)
				;
		}
		else if(ucSystemPowerMode == 0x01)
		{
			uiValue = uiValue/10;
			T0PR = 5-1; //1000 Hz
			T0TCR = 0x1;
		
			T0PC = 0;
			T0TC = 0;
			
			while(T0TC <= uiValue)
				;
		}
		else
		{
			T0PR = 4-1; //4mhz pclk
			T0TCR = 0x1;
		
			T0PC = 0;
			T0TC = 0;
			
			while(T0TC <= uiValue)
				;
		}
	}
	else
	{
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");//
	}
}

//ADC0
/*void cTxos::vADC0PowerEnable()
{
	AD0CR_bit.PDN = 0;
	while(AD0CR_bit.PDN)
		;
	asm("NOP");
	asm("NOP");
	PCONP_bit.PCAD = 1;
}

void cTxos::vADC0PowerDisable()
{
	AD0CR_bit.PDN = 0;
	while(AD0CR_bit.PDN)
		;
	asm("NOP");
	asm("NOP");
	PCONP_bit.PCAD = 0;
}*/

void cTxos::vADC0Enable(unsigned char ucCh)
{
	PCONP_bit.PCAD = 1;
	
	switch(ucCh)
	{
	  case 0://p0_23
		PINSEL1 |= 0x00004000;
		PINSEL1 &= 0xffff7fff;
		break;
	  case 1://p0_24
		PINSEL1 |= 0x00010000;
		PINSEL1 &= 0xfffdffff;
		break;
	  case 2://p0_25
		PINSEL1 |= 0x00040000;
		PINSEL1 &= 0xfff7ffff;
		break;
	  case 3://p0_26
		PINSEL1 |= 0x00100000;
		PINSEL1 &= 0xffdfffff;
		break;
	  case 4://p1_30
		PINSEL3 |= 0x30000000;
		PINSEL3 &= 0xffffffff;
		break;
	  case 5://p1_31
		PINSEL3 |= 0xc0000000;
		PINSEL3 &= 0xffffffff;
		break;
	  case 6://p0_3
		PINSEL0 |= 0x00000080;
		PINSEL0 &= 0xffffffbf;
		break;
	  case 7://p0_2
		PINSEL0 |= 0x00000020;
		PINSEL0 &= 0xffffffef;
		break;
	  default:
		;
	}
}

void cTxos::vADC0Disable(unsigned char ucCh)
{
	AD0CR_bit.START = 0;
	
	switch(ucCh)
	{
	  case 0://p0_23
		PINSEL1 &= 0xffff3fff;
		break;
	  case 1://p0_24
		PINSEL1 &= 0xfffcffff;
		break;
	  case 2://p0_25
		PINSEL1 &= 0xfff3ffff;
		break;
	  case 3://p0_26
		PINSEL1 &= 0xffcfffff;
		break;
	  case 4://p1_30
		PINSEL3 &= 0xcfffffff;
		break;
	  case 5://p1_31
		PINSEL3 &= 0x3fffffff;
		break;
	  case 6://p0_3
		PINSEL0 &= 0xffffff3f;
		break;
	  case 7://p0_2
		PINSEL0 &= 0xffffffcf;
		break;
	  default:
		;
	}
}

unsigned short cTxos::usADC0Read(unsigned char ucCh, unsigned char usFreq)
{
	AD0CR = (0x00200000|(1<<ucCh)|(usFreq<<8));
	usFreq = AD0GDR;
	AD0CR_bit.START = 1;
	
	while(!AD0GDR_bit.DONE)
		;
	
	AD0CR_bit.START = 0;
	
	return ((AD0GDR>>4)<<0);//_bit.RESULT;
}

//DAC
void cTxos::vDACEnable()
{
	PINSEL1  |= 0x00200000;
	PINSEL1  &= 0xffefffff;
	PINMODE1 |= 0x00200000;
	PINMODE1 &= 0xffefffff;
	
	vDelayUS(1000);
	DACR_bit.BIAS = 0; //1:400khz 0:1mhz
}

void cTxos::vDACDisable()
{
	PINSEL1  &= 0xffcfffff;
}

//UART
void cTxos::vUartEnable(unsigned char ucMode) //MODE 0:1 channal - 2:7 baud rate
{
	unsigned char ucCh = (ucMode & 0x03),
	              ucDLL = 4,//2
				  ucDLM = 0,
				  ucDIVADDVAL = 2,//1
				  ucMULVAL = 9;//12
	
	switch(ucCh)
	{
	  case 1:
		PCONP_bit.PCUART1 = 1;
		U1LCR_bit.DLAB = 1;
		U1DLL = ucDLL;
		U1DLM = ucDLM;
		U1LCR_bit.DLAB = 0;
		U1FDR_bit.DIVADDVAL = ucDIVADDVAL;
		U1FDR_bit.MULVAL = ucMULVAL;
		vDelayUS(10000);
		U1TER_bit.TXEN = 1;
		U1LCR = 0x03;
		U1FCR = 0x07;
		PINSEL0  |= 0x40000000;
		PINSEL0  &= 0x7fffffff;
		PINSEL1  |= 0x00000001;
		PINSEL1  &= 0xfffffffd;
		
		PINMODE0 |= 0x80000000;
		PINMODE0 &= 0xbfffffff;
		PINMODE1 |= 0x00000b2a;
		PINMODE1 &= 0xfffffbea;
		break;
	  case 2:
		PCONP_bit.PCUART2 = 1;
		U2LCR_bit.DLAB = 1;
		U2DLL = ucDLL;
		U2DLM = ucDLM;
		U2LCR_bit.DLAB = 0;
		U2FDR_bit.DIVADDVAL = ucDIVADDVAL;
		U2FDR_bit.MULVAL = ucMULVAL;
		vDelayUS(10000);
		U2TER_bit.TXEN = 1;
		U2LCR = 0x03;
		PINSEL0  |= 0x00500000;
		PINSEL0  &= 0xff5fffff;
		PINMODE0 |= 0x00a00000;
		PINMODE0 &= 0xffafffff;
		break;
	  default:
		;
	}
}

void cTxos::vUartDisable(unsigned char ucMode) //MODE 0:1 channal
{
	;
}

void cTxos::vUartReset(unsigned char ucMode) //MODE 0:1 channal - 2 buffer - 3 tx reset - 4 rx reset
{
	unsigned char ucCh = (ucMode & 0x03),
	              ucRST = ((ucMode >> 2)&0x7);
	
	switch(ucCh)
	{
	  case 0:
		U0FCR = ucRST;
		break;
	  case 1:
		U1FCR = ucRST;
		break;
	  case 2:
		U2FCR = ucRST;
		break;
	  default:
		U3FCR = ucRST;
	}
}

void cTxos::vUartSend(unsigned char ucCh,char const* ucData)
{
	unsigned int  ucI;
	
	if(ucCh == 0)
	{
		for(ucI=0; ucI<strlen(ucData); ucI++)
			U0THR = ucData[ucI];
	}
	else if(ucCh == 1)
	{
		for(ucI=0; ucI<strlen(ucData); ucI++)
		{
			U1THR = ucData[ucI];
			
			while(U1LSR_bit.THRE == 0)
				;
		}
	}
	else if(ucCh == 2)
		for(ucI=0; ucI<strlen(ucData); ucI++)
			U2THR = ucData[ucI];
	else
		for(ucI=0; ucI<strlen(ucData); ucI++)
			U3THR = ucData[ucI];
}

bool cTxos::bUartReceive(unsigned char ucCh)
{
	unsigned int	uiI;
	unsigned char	ucX;
	
	vDelayUS(1000);
	
	if(ucCh == 0)
		;
	else if(ucCh == 1)
	{		
		while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
			;
		
		for(uiI=0;uiI<10000;uiI++)
		{
			vDelayUS(100);
			
			if(U1LSR & 0x01)
			{
				uiI = 0;
				ucX = U1RBR;
				
				if(ucX == 0x0a)
					;
				else if(ucX == 0x0d)
					;
				else if(ucX<128 && ucX>6)
					;
				else
					;
			}
			else
				U1FCR = 0x07;
		}
		vDelayUS(1000);
	}
	else if(ucCh == 2)
		;
	else
		;
	
	return 0;
}

void cTxos::vWaitForUartResponseMS(unsigned char ucCh, unsigned int uiTime)
{
	//just uart 1:
	unsigned char ucX, ucJ=0, ucY=0;
	
	ucDataRam[0][0]='\0';
	ucDataRam[1][0]='\0';
	ucDataRam[2][0]='\0';
	ucDataRam[3][0]='\0';
	ucDataRam[4][0]='\0';
	ucDataRam[5][0]='\0';
	ucDataRam[6][0]='\0';
	ucDataRam[7][0]='\0';
	
	while(U1LSR_bit.TEMT == 0 || U1LSR_bit.THRE == 0)
		;
		
	for(uiTime*=10;uiTime!=0;uiTime--)
	{
		vDelayUS(100);
		
		if(U1LSR & 0x01)
		{
			ucX = U1RBR;

			if(ucX == 0x0a || ucX == 0x0d)
			{
				if(ucDataRam[ucY][0] != '\0')
					ucY++;
			}
			else if(ucX<128 && ucX>6)
			{
				ucJ = strlen(ucDataRam[ucY]);
				ucDataRam[ucY][ucJ] = ucX;
				ucDataRam[ucY][ucJ+1] = '\0';
			}
			else
				;
		}
	}
}

//BUZZER vBuzzMS(500, 250, 0x3ff);
void cTxos::vBuzzMS(unsigned int uiTime, unsigned int uiFreq, unsigned int uiVolume)
{
	//FIO3DIR_bit.P3_26=1;//Speaker on
	//FIO3DIR_bit.P3_26=0;//Speaker off
	vDACEnable();
	uiTime*=2;
	
	for(; uiTime; uiTime--)
	{
		DACR_bit.VALUE = uiVolume;//10 bits
		vDelayUS(uiFreq);
		
		DACR_bit.VALUE = 0x000;//10 bits
		vDelayUS(uiFreq);
	}
	vDACDisable();
}
