/*************************************************************************
 *
ucSystemMode bits:
0-1: System status
	00: off (Put sim908 on AT+CFUN=0)
	01: on (Load BG, PhB, main menu & welcome)
	10: Reset (Load BG & main menu agin)
	11: normal
2: Sim908
	0: Sim908 disable
	1: Sim908 enable
3-4: Battery
	00: //Empty
	01: //Low
	10: //Full
	11: //Plugin
5-7: Softwares
	call:
	000: GSM mode
	menu:
	001: //SMS mode
	010: GPS mode
	011: MPX mode
	100: //External Software 1
	101: //External Software 2
	110: //External Software 3
	111: Reserved
	
ucSim908Mode bits:
	0: enable
	1: gsm on
	2: call ready
	3: op name
	4: gps on
	5: gps 2d fix
	6: gps 3d fix
	7: sleep

===================
GSM mode:
	SW Mode:
		0: init
		1: main page (desktop)
		2: menu
		3: phone book		Without TS
		4: phone			TS for numbers
		5: calling			Without TS
		6: incoming call	Without TS
SMS mode:
	SW Mode:
		0: init
		1: -
GPS mode:
	SW Mode:
		0: init
		1: options
MPX mode:
	SW Mode:
		0: init
		1: options

 *
 **************************************************************************/
#include "includes.h"

class cTxos
{
  public:
	//****************************************
	//********** Public variables: ***********
	//****************************************	
	char 			ucFileBlockData[512],		//Data block 512 byte
					ucFileName[9],				//file name + \0
					ucFileType[4],				//file type + \0
					ucFileDir[512],				//Opened Directory data (included 16 file or directory max)
					ucFileSPC,					//sectors per cluster
					ucFileFAT,					//number of fats
					ucFileLastBlockOfCal,		//The last data block of the last file caluster that read
					ucFileLastCal,				//The last caluster of file that read
					ucFileMaxBlocksOfCals,		//Number of data blocks on each caluster
					ucSystemLog[100],			//System log + \0
					ucDataRam[8][512],			//Data ram
					ucTimerMode,				//
					ucSim908Mode,				//
					ucSystemMode,				//
					ucLastChar,					//
					ucFontSize,					//
					ucErrMsg[21],				//
					ucSWMode,					//
					ucGPSLat[13],				//
					ucGPSLong[13],				//
					ucGPSSpeed[13],				//
					ucPhoneBook[10][10],		//
					ucPhoneBookNumber[10][15],	//
					ucIncomingCalls[6][16],		//
					ucIncomingCallN,			//
					ucPhoneBookN,				//
					ucOp,						//
					ucPhoneNumber[21],			//
					ucGPSResetMode,				//
					ucSystemPowerMode,			//
					ucSMSPrintCharNumber,		//
					ucSMSPrintCharPos,			//
					ucSMSPrintNumberPos;		//

	unsigned short	usFileOSR,				//file offset sectors from root
					usFileBPS,				//bytes per sector
					usFileFRS,				//first reserved sectors
					usFileTRE,				//total root entries
					usFileSPF,				//sectors per fat
					usFileEntryOffset,		//Offset of entry of file, from root directory
					usFileNTS,				//number of total sectors
					usFileCalusters[256],	//File pointer
					usFileROOTFolderOSR,	//
					usFileGPSFolderOSR,		//
					usFileMPXFolderOSR,		//
					usFileMENUOSR,			//
					usFileBGOSR,			//
					usFilePHBOSR,			//
					usFileNOSR,				//
					usFileSMSOSR[3],		//
					usFileGPSOSR,			//
					usFileMAPOSR,			//
					usFileMPXOSR,			//
					usFontCharPos,			//
					usFontBackground,		//
					usFontColor,			//
					usTouchXMax,			//
					usTouchYMax,			//
					usTouchXMin,			//
					usTouchYMin,			//
					usTouchX,				//
					usTouchY;				//
	
	unsigned int	uiLCDBLCounter,			//
					uiIntLongitude,			//
					uiIntLatitude,			//
					uiMapPxLo,				//
					uiMapPxLa,				//
					uiMapNuOLo,				//number of long maps
					uiMapNuOLa,				//
					uiMapLongStart,			//
					uiMapLongEnd,			//
					uiMapLatStart,			//
					uiMapLatEnd,			//
					uiMapLo,				//last Drawn long map
					uiMapLa,				//
					uiCurrentLoationPxLo,	//
					uiCurrentLoationPxLa,	//
					uiMPXCounter,			//
					uiMPXCount;				//
	
	bool			bCalling,		//
					bMPXPlay,		//
					bNewCall,		//
					bNewSMS,		//
					bCallStatus,	//
					bRingInit,		//
					bRTL,			//
					bNCSelect;		//

	//****************************************
	//********** System functions: ***********
	//****************************************
	//CLOCK&POWER Modes
	void			vHighSpeedOperation();//18MHz
	void			vNormalOperation();//4MHz
	void			vSleep();//512MHz
	void			vPowerDown();//15KHz Flash/6
	//GPIO
	void			vGpioInit();
	//TIMER
	void			vDelayUS(unsigned int uiValue);
	//ADC0
	//void			vADC0PowerEnable();
	//void			vADC0PowerDisable();
	void			vADC0Enable(unsigned char ucCh);
	void			vADC0Disable(unsigned char ucCh);
	unsigned short	usADC0Read(unsigned char ucCh, unsigned char usFreq);
	//DAC
	void			vDACEnable();
	void			vDACDisable();
	/*void			usDACWrite(unsigned short usValue);==>DACR_bit.VALUE = value; //10 bits*/
	//UART
	void			vUartEnable(unsigned char ucMode);
	void			vUartDisable(unsigned char ucMode);
	void			vUartReset(unsigned char ucMode);
	void			vUartSend(unsigned char ucCh,char const* ucData);
	bool			bUartReceive(unsigned char ucCh);
	void			vWaitForUartResponseMS(unsigned char ucCh, unsigned int uiTime);
	//BUZZER
	void			vBuzzMS(unsigned int uiTime,
							unsigned int uiFreq,
							unsigned int uiVolume);
	//****************************************
	//******** LCD & Font functions: *********
	//****************************************
	void     		vLCDInit();
	void			vLCDClrScr();
	
	void 			vLCDSendData(unsigned short usAddr, unsigned short usData);
	
	unsigned short	usLCDReceiveData(unsigned short usAddr);
	
	void			vPrintPixel(unsigned short usX,
								unsigned char  ucY,
								unsigned short usColor);

	void			vPrint(unsigned char ucChar);
	void 			vPrint2(unsigned char  ucChar,
				 			unsigned short usCharPos,
				 			unsigned short usBackground,
				 			unsigned short usColor,
				 			unsigned char  ucSize);
	
	void 			vSPrint(char const* usString);
	void 			vSPrint2(char     const* usString,
				  			 unsigned short  usCharPos,
				  			 unsigned short  usBackground,
				  			 unsigned short  usColor,
				  			 unsigned char   ucSize);
	
	void 			vSetPBGCS(unsigned short usCharPos,
							  unsigned short usBackground,
				   			  unsigned short usColor,
				   			  unsigned char  ucSize);
	
	void 			vPrintNewLine();
	void 			vDetailsLineClr(unsigned char ucLine);
	
	unsigned char	ucSystemUTFPersianConvert(unsigned char ucChar, unsigned char ucNextChar);
	
	void			vPrintALLAH();
	//****************************************
	//******* Touch Screen functions: ********
	//****************************************
	void 			vTouchCal();
	void 			vTouchGetXY();
	
	void			vTouchGetPoint();//usTouchX:0-319 usTouchY:0-239
	void			vTouchGetOperation();
	
	void			vTouchGetKeyboard();
	//****************************************
	//********** Sim908 functions: ***********
	//****************************************
	void 			vSim908OnOff();
	bool			bSim908SendCMD(char const* ucData, unsigned char ucRe=0, unsigned short usWFR=2000);
	bool			bSim908CheckMode();
	
	bool		 	bSim908GsmOn();
	bool 			bSim908GsmOff();
	bool			bSim908GetRegStatus();
	bool			bSim908Call(char const* ucData);
	bool			bSim908GetCallingStatus();
	
	bool 			bSim908GpsOn();
	bool 			bSim908GpsOff();
	bool			bSim908GetGPSStatus();
	bool			bSim908SetCall();
	bool			bSim908Sleep();
	bool			bSim908WakeUp();
	
	bool			bSim908HangUpCall();
	bool			bSim908AnswerCall();
	
	bool			bSim908CheckResponce(char const* ucData);
	unsigned char	ucSim908CheckResponceCMD(char const* ucData);
	
	void			vSim908ShowTimeBATAnt();
	
	bool			bSim908SendSMS();
	bool			bSim908ReadSMS();
	//****************************************
	//**********   FAT functions:  ***********
	//****************************************
	bool 			bFileInit();
	
	bool 			bOpenDir(unsigned short usCAddr);
	void 			vReadDir();
	
	bool 			bOpenFile(unsigned short usOffsetCaluster);
	bool 			bReadFile();
	
	bool			bWriteFile();
	
	void			vReadFileGoToAddr(unsigned int uiAddr);
	//****************************************
	//********** SD-MMC functions: ***********
	//****************************************
	bool 			bSDMMCInit();
	void 			vSDMMCExtraClocks(unsigned char ucTime);
	
	bool 			bSDMMCSetBlockLengh(unsigned int uiLengh);
	void 			vSDMMCSendCommand(unsigned char ucCmd, unsigned int uiAr, unsigned char ucCrc);
	void 			vSDMMCSendByte(unsigned char ucByte);
	bool 			bSDMMCSendBlock(unsigned int uiAddr);//ucSDMMCBlockData[0:511] ==> send
	
	bool 			bSDMMCGetByte(unsigned char ucByte);
	
	unsigned char 	ucSDMMCReceiveByte();
	bool 			bSDMMCReceiveBlock(unsigned int uiAddr);//receive ==> ucFileBlockData[0:511]
	//****************************************
	//**********  FILE functions:  ***********
	//****************************************
	bool 			bDrawBMPFile(unsigned short usCalNumber);
	bool 			bGetNuFromPhBook(unsigned short usCalNumber);
	bool			bDrawSMSBMPFile();
	//****************************************
	//*The Xerxes Operating System functions:*
	//****************************************
	void			vOSStart();
	void 			vOSLoad();
	void			vOSMainMenuLoad();
	
	void			vOSCheckUART();
	void			vOSCheckInputs();
	
	bool 			bOSFileInit();
	
	void			vOSCalling();
	
	void			vOSError();
	void			vOperation();
	
	void			vCallListRun();
	//****************************************
	//******* GPS Software functions: ********
	//****************************************
	void			vLoadGPSSoftware();
	
	bool			bLoadPointFromGPSMap();
	bool			bMapFileStatus(unsigned short usCalNumber);
	bool			bMapDraw();
	void			vMapPrintCurrentLacation();
	
	void			vLocationAToUI();
	//****************************************
	//******* MPX Software functions: ********
	//****************************************
	void			vMPXSoftwareRun();
	void			vMPXPlayBuffer();
	
	bool			bReadMPXFile();
	//****************************************
	//******* SMS Software functions: ********
	//****************************************
	void			vSMSSoftwareRun();
	void			vSMSSoftwareTextBox();
	void			vSMSSoftwareInBox();
	void			vSMSSoftwareLoadMessage();
	
	unsigned char	ucITOHexFAConvert(unsigned char ucCh);
	unsigned char	ucHexTOIFAConvert(unsigned char ucCh);
	unsigned char	ucHexToI(unsigned int uiHex);
	unsigned int	uiITOHex(unsigned char ucCh);
	//****************************************
	//******* VLM Software functions: ********
	//****************************************
	void			vVLMSoftwareRun();
} ;
